;Colour bars program
;-------------------
;Binary created using DAsm 2.12 running on an Amiga.

   PROCESSOR   6502

   ORG   $E000    ;16Kb PRG-ROM, 32Kb CHR-ROM

   dc.b  #$00,#$01,#$02,#$03

Reset_Routine  SUBROUTINE
   cld         ;Clear decimal flag
   sei         ;Disable interrupts
.WaitV   lda $2002
   bpl .WaitV     ;Wait for vertical blanking interval
   ldx #$00
   stx $2000
   stx $2001      ;Screen display off, amongst other things
   dex
   txs         ;Top of stack at $1FF

;Clear (most of) the NES' WRAM. This routine is ripped from "Duck Hunt" - I should probably clear all
;$800 bytes.
   ldy #$06    ;To clear 7 x $100 bytes, from $000 to $6FF?
   sty $01        ;Store count value in $01
   ldy #$00
   sty $00
   lda #$00

.Clear   sta ($00),y    ;Clear $100 bytes
   dey
   bne .Clear

   dec $01        ;Decrement "banks" left counter
   bpl .Clear     ;Do next if >= 0

   lda   #$20
   sta   $2006
   lda   #$00
   sta   $2006

   ldx   #$00
   ldy   #$10
.ClearPPU sta $2007
   dex
   bne   .ClearPPU
   dey
   bne   .ClearPPU

   lda   #$00
   sta   $2003
   ldy   #$00
.ClearSprites sta $2004
   dey
   bne   .ClearSprites


;********* Initialize Palette to specified colour ********

   ldx   #$3F
   stx   $2006
   ldx   #$00
   stx   $2006

   ldx   #$00
   ldy   #$10     ;Clear BG palettes.
.InitPal lda   .Palette,X
   sta   $2007
   inx
   dey
   bne   .InitPal
;*********************************************************

   lda   #$20
   sta   $2006
   lda   #$00
   sta   $2006       ;Point 0 rows down in Name Table.

   ldy   #$03
   ldx   #$00
.NAMMap1
   stx   $2007
   inx
   bne   .NAMMap1
   dey
   bne   .NAMMap1

   ldx   #$00
.NAMMap2
   stx   $2007
   inx
   cpx   #$C0
   bne   .NAMMap2

;------------------------------
;Set up Attribute Table
;   lda   #$23
;   sta   $2006
;   lda   #$C0
;   sta   $2006       ;Point to right spot in Attr. Map
;
   ldx   #$00
   ldy   #$40
.DrawAtt
   lda   .AttMap,X
   sta   $2007
   inx
   dey
   bne   .DrawAtt

;------------------------------
;Set up Sprite
   lda   #$00
   sta   $2003    ;Point to Sprite 0
   lda   #$09
   sta   $2004
   lda   #$01
   sta   $2004
   lda   #%00000000
   sta   $2004
   lda   #$00
   sta   $2004

;-------------------------------
   lda   #$00
   sta   $2006
   sta   $2006       ;Reset PPU
   sta   $2005
   sta   $2005       ;Reset Scroll

   lda   #$00
   sta   $E000       ;Reset Mapper.
   sta   $10

;Enable vblank interrupts, etc.
   lda   #%10000000
   sta   $2000
   lda   #%00011110   ;Screen on, sprites on, show leftmost 8 pixels, colour
   sta   $2001
;  cli            ;Enable interrupts(?)

   lda   #$00
   sta   $40      ;Store no VB flag

;Now just loop forever?
.Loop

   lda   #$01
   cmp   $40
   bne   .Loop
.CheckSpr1 lda $2002
   and   #$40          ;Check for Sprite Clear (!)
   bne   .CheckSpr1
.CheckSpr2 lda $2002
   and   #$40
   beq   .CheckSpr2

   ldy   #$05
   ldx   #$C0

.TimingLoop1
   dex
   bne   .TimingLoop1
   dey
   bne   .TimingLoop1

   lda   #%10010000
   sta   $2000
   lda   #$00
   sta   $40
   lda   $2002

   ldy   #$05
   ldx   #$B6

.TimingLoop2
   dex
   bne   .TimingLoop2
   dey
   bne   .TimingLoop2

   lda   #%10000000
   sta   $2000
   lda   $10
   ora   #$01
   tax
   sta   $E000,X

   ldy   #$06
   ldx   #$AA

.TimingLoop3
   dex
   bne   .TimingLoop3
   dey
   bne   .TimingLoop3

   lda   #%10010000
   sta   $2000

.NoCheck
   jmp   .Loop

.AttMap INCLUDE FDSPic.ATT

.Palette    dc.b  #$10,#$00,#$07,#$17,#$10,#$02,#$0D,#$17
            dc.b  #$10,#$07,#$16,#$20,#$10,#$02,#$0D,#$20

NMI_Routine SUBROUTINE
   pha
   txa
   pha
   tya
   pha
   php

   lda   $10
   eor   #$02
   sta   $10
   tax
   sta   $E000,X

   lda   #%10000000
   sta   $2000
   lda   #$01
   sta   $40         ;Stores VBlank flag (for me)

   lda   #$00
   sta   $2005
   sta   $2005

   plp
   pla
   tay
   pla
   tax
   pla
   rti

IRQ_Routine       ;Dummy label
   rti

;That's all the code. Now we just need to set the vector table approriately.

   ORG   $FFFA,0
   dc.w  NMI_Routine
   dc.w  Reset_Routine
   dc.w  IRQ_Routine    ;Not used, just points to RTI


;The end.
